export default class ChecklistForm {
  constructor(config) {
    this.config = config;
    this.form = document.getElementById(config.formId);
    this.isAddress = Array.isArray(config.fieldId);
    this.saveButton = document.getElementById(`btn-save-${this.config.interpolation.id}`);
    this.signaturePad;
    if (!this.isAddress) {
      this.field = document.getElementById(config.fieldId);
    }

    if (this.form) {
      this.setFormEventListener();
      document.addEventListener("DOMContentLoaded", () => {
        this.loadCustomFieldAnswer();
        this.limitImageSize();
      });
      this.handleSignature();
    }
  }

  setFormEventListener() {
    this.form.addEventListener("submit", async (e) => {
      e.preventDefault();

      if (this.saveButton) {
        this.saveButton.style.pointerEvents = 'none';
        this.saveButton.style.cursor = 'wait';
        this.saveButton.textContent = 'Carregando...';
      }

      const params = new URLSearchParams(window.location.search);
      const dealIdParam = params.get("dealId");
      const secret = "V3d2rWwspFU^#jBz";

      const decryptedBytes = CryptoJS.AES.decrypt(dealIdParam, secret);
      const dealId = decryptedBytes.toString(CryptoJS.enc.Utf8);

      let answer = this.field?.value;

      if (this.config.fieldId?.includes("GET_CHECKLIST_ANSWER")) {
        const fieldId = answer.split("-")[1];
        const section = document.querySelector(`#checklist-${fieldId}`);
        if (section) {
          answer = Array.from(
            section.querySelectorAll('input[name="checklist"]:checked')
          )
            .map((input) => input.value)
            .join(", ");
        }
      }

      const token = "0b1722c694e40df1b324c311617c4a8e";

      let body = {
        projetoId: dealId,
        resposta: this.handleAnswerValue(answer),
        token,
      };

      let apiUrl = this.config.apiUrl + this.config.endpoint;
      let method = "POST";
      const isNotCustomField = apiUrl?.toLowerCase().includes("/projeto") ||
        apiUrl?.toLowerCase().includes("/lead/porprojeto");

      if (this.isAddress) {
        const id = this.config.interpolation.id;
        const address = {
          logradouro: document.getElementById(`endereco-rua-${id}`)?.value,
          numero: document.getElementById(`endereco-numero-${id}`)?.value,
          complemento: document.getElementById(`endereco-complemento-${id}`)?.value,
          bairro: document.getElementById(`endereco-bairro-${id}`)?.value,
          cidade: document.getElementById(`endereco-cidade-${id}`)?.value,
          uf: document.getElementById(`endereco-estado-${id}`)?.value,
          cep: document.getElementById(`endereco-cep-${id}`)?.value,
        };
        body = { ...address, token };
        apiUrl += `/${dealId}?porProjetoId=true`;
        method = 'PUT';
      } else if (isNotCustomField) {
        body = {
          propriedade: this.config.property,
          valor: this.handleAnswerValue(answer),
        };
        apiUrl += `/${dealId}?token=${token}`;
        method = "PUT";
      } else if (this.config.fieldId.includes('assinatura-') && this.signaturePad) {
        const dataUrl = this.signaturePad.toDataURL();
        const blob = this.dataURLToBlob(dataUrl);
        const images = await this.uploadImage(blob, `${this.config.property}-${dealId}.png`);
        body.resposta = this.handleImageAnswer(images);
      } else if (this.config.fieldId.includes('imagem-') || this.config.fieldId.includes('imagens-')) {
        const files = document.getElementById(this.config.fieldId)?.files;
        const filesArray = Array.from(files);
        const uploadPromises = filesArray.map((file) => this.uploadImage(file));
        const images = (await Promise.all(uploadPromises)).flat();
        body.resposta = this.handleImageAnswer(images);
      }

      try {
        const res = await fetch(apiUrl, {
          body: JSON.stringify(body),
          method,
          headers: {
            "Content-Type": "application/json",
          },
        });

        const data = await res.json();

        let newValue = data.Content.resposta;

        if (this.isAddress) {
          const { logradouro, numero, bairro, cidade, uf } = data.Content.endereco;
          newValue = `${logradouro}, ${numero}, ${bairro}, ${cidade} - ${uf}`;
        } else if (isNotCustomField && !!this.config.property) {
          newValue = data.Content[String(this.config.property).toLowerCase()];
        }

        this.loadCustomFieldAnswer(newValue);
      } catch (error) {
        console.log(error);
      } finally {
        if (this.saveButton) {
          this.saveButton.style.pointerEvents = 'all';
          this.saveButton.style.cursor = 'pointer';
          this.saveButton.textContent = 'Salvar';
        }
      }
    });
  }

  handleAnswerValue(value = '') {
    if (!isNaN(Number(value.replace(',', '.')))) {
      return Number(value).toString().replace('.', ',');
    }
    return value;
  }

  interpolationHasValue(value = '') {
    const isHifen = String(value).replaceAll(" ", "").replaceAll("  ", "") === "-";
    const isSemicolon = String(value).replaceAll(" ", "").replaceAll("  ", "") === ";";
    const isZero = Number(value.replace(',', '.')) === 0;

    return value && !isHifen && !isZero && !isSemicolon;
  }

  loadCustomFieldAnswer(answer = '') {
    const { interpolacao, id } = this.config.interpolation;
    if (this.interpolationHasValue(interpolacao) || this.interpolationHasValue(answer)) {
      const container = document.getElementById(`form-container-${id}`);
      const form = document.getElementById(`form-${id}`);

      const buttonGroup = document.getElementById(`button-group-${id}`);
      if (buttonGroup) buttonGroup.remove();

      const fieldId = this.isAddress ? this.config.fieldId[0] : this.config.fieldId;
      const label = document.getElementById(fieldId).previousElementSibling;

      let interpolation;

      if (this.config.fieldId.includes('assinatura-') || this.config.fieldId.includes('imagem-')) {
        interpolation = document.createElement("div");
        const imageValue = this.interpolationHasValue(interpolacao) ? interpolacao : answer;
        const newElement = document.createElement(this.isImage(imageValue) ? 'img' : 'div');
        if (this.isImage(imageValue)) {
          newElement.src = imageValue;
          newElement.style.maxWidth = '100%';
        } else {
          const anchor = document.createElement('a');
          anchor.href = item;
          anchor.textContent = item;
          anchor.target = "_blank";
          anchor.rel = "noopener noreferrer";
          
          newElement.appendChild(anchor);
        }
      
        interpolation.appendChild(newElement);
      } else if (this.config.fieldId.includes('imagens-')) {
        interpolation = document.createElement("div");
      
        if (answer) {
          const imageUrls = answer.split(';');
      
          imageUrls.forEach((item) => {
            const element = document.createElement(this.isImage(item) ? 'img' : 'div');
            if (this.isImage(item)) {
              element.src = item;
              element.style.maxWidth = '100%';
            } else {
              const anchor = document.createElement('a');
              anchor.href = item;
              anchor.textContent = item;
              anchor.target = "_blank";
              anchor.rel = "noopener noreferrer";
              
              element.appendChild(anchor);
            }
      
            interpolation.appendChild(element);
          });
        } else {
          interpolation.innerHTML = this.interpolationHasValue(interpolacao) ? interpolacao : answer;
        }
      } else {
        interpolation = document.createElement("div");
        interpolation.textContent = this.interpolationHasValue(interpolacao) ? interpolacao : answer;
      }

      if (container) {
        form.remove();
        container.appendChild(label);
        container.appendChild(interpolation);
      }
    }
  }

  handleSignature() {
    const canvas = this.field;
    const clear = document.getElementById(`btn-clear-${this.config.interpolation.id}`);

    this.signaturePad = new SignaturePad(canvas);
    if (this.signaturePad) {
      clear.addEventListener('click', () => {
        this.signaturePad.clear();
      });
      
      function resizeCanvas() {
        const ratio =  Math.max(window.devicePixelRatio || 1, 1);
        canvas.width = canvas.offsetWidth * ratio;
        canvas.height = canvas.offsetHeight * ratio;
        canvas.getContext("2d").scale(ratio, ratio);
      }
      
      window.addEventListener("resize", resizeCanvas);
      resizeCanvas();
    }
  }

  async uploadImage(file, fileName) {
    try {
      const formData = new FormData();
      formData.append('file', file, fileName);

      const res = await fetch(this.config.apiUrl + '/arquivo', {
        method: 'POST',
        body: formData,
      });
      const data = await res.json();
      return data.Content;
    } catch (error) {
      console.log(error);
    }
  }

  dataURLToBlob(dataURL) {
    const byteString = atob(dataURL.split(',')[1]);
    const mimeString = dataURL.split(',')[0].split(':')[1].split(';')[0];
  
    const ab = new ArrayBuffer(byteString.length);
    const ia = new Uint8Array(ab);
    for (let i = 0; i < byteString.length; i++) {
      ia[i] = byteString.charCodeAt(i);
    }
  
    return new Blob([ab], { type: mimeString });
  }

  handleImageAnswer(images) {
    const urls = images.map((item) => item.url);
    return urls.join(';');
  }

  isImage(src) {
    const imageExtensions = ['.jpg', '.jpeg', '.png', '.gif', '.bmp', '.webp', '.svg'];
    return imageExtensions.some(ext => src.toLowerCase().includes(ext));
  }

  limitImageSize() {
    if (this.config.fieldId.includes('imagem-') || this.config.fieldId.includes('imagens-')){
      const MAX_SIZE = 10 * 1024 * 1024;
      this.field.addEventListener('change', () => {
        const files = this.field.files;
        let oversized = false;
        
        for (let i = 0; i < files.length; i++) {
          if (files[i].size > MAX_SIZE) {
            oversized = true;
            break;
          }
        }

        if (oversized) {
          alert('Arquivo não pode exceder 10MB.');
          this.field.value = "";
        } else {
          oversized = false;
        }
      });
    }
  }
}
